package tools.descartes.dql.connector.kieker;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

import tools.descartes.dql.connector.AbstractOSGiQueryConnector;
import tools.descartes.dql.connector.ModelStructureQueryConnector;
import tools.descartes.dql.connector.QueryProgressListener;
import tools.descartes.dql.connector.kieker.structures.Trace;
import tools.descartes.dql.connector.kieker.structures.records.EventRecord;
import tools.descartes.dql.connector.kieker.structures.records.ResourceRecord;
import tools.descartes.dql.core.engine.util.DQLLogger;
import tools.descartes.dql.models.mapping.domain.DomainFactory;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.impl.DomainFactoryImpl;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class ModelStructureQueryConnectorImpl extends AbstractOSGiQueryConnector
		implements ModelStructureQueryConnector {
	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	private final DomainFactory domainFactory = DomainFactoryImpl.init();

	public ModelStructureQueryConnectorImpl() {
		super();
	}

	/**
	 * Creates and attaches probes to a list of probes
	 * 
	 * @param metrics
	 *            Metrics to be attached as probes
	 * @param probes
	 *            List of probes, where probes for metrics shall be attached
	 */
	private void attachProbes(final String[] metrics, final EList<Probe> probes) {
		Probe probe = null;

		for (final String metric : metrics) {
			probe = domainFactory.createProbe();
			probe.setMetricName(metric);

			probes.add(probe);
		}
	}

	@Override
	public String getBundleProperty(final String key) {
		return Activator.getContext().getProperty(key);
	}


	 


	/**
	 * Attaches available probes to resources
	 * 
	 * @param domainFactory
	 * @param resources
	 */
	// private void processProbesForResources(final EList<Resource> resources) {
	// // TODO support DMLHelper.METRIC_THROUGHPUT also for resources
	// final String[] metrics = new String[] { KiekerHelper.METRIC_UTILIZATION
	// };
	//
	// for (final Resource r : resources) {
	// attachProbes(metrics, r.getProbes());
	// }
	// }

	/**
	 * Attaches available probes to services
	 * 
	 * @param domainFactory
	 * @param resources
	 */
	// private void processProbesForServices(final EList<Service> services) {
	// final String[] metrics = new String[] {
	// KiekerHelper.METRIC_RESPONSE_TIME_AVG, KiekerHelper.METRIC_THROUGHPUT };
	//
	// for (final Service s : services) {
	// attachProbes(metrics, s.getProbes());
	// }
	// }

	/**
	 * Process a ProcessingResourceSpecification and add it to the EntityMapping
	 * list of resources, the alias is enriched by a specific prefix in order to
	 * understand where the ProcessingResource comes from.
	 * 
	 * @param resources
	 *            The EntityMapping list of resources
	 * @param identifier
	 *            The DML Identifier of the resource to be modified
	 * @param namedElement
	 *            The DML NamedElement of the resource to be modified
	 * @return The created EntityMapping representation
	 */
	// private void processProcessingResourceSpec(
	// final EList<tools.descartes.dql.models.mapping.domain.Resource>
	// resources,
	// final ProcessingResourceSpecification prs) {
	// // compute understandable alias as concatenation of Strings of names of
	// // Container -> ProcessingResourceSpecification
	// String alias = null;
	// EObject parent = prs.eContainer();
	// if (parent instanceof Container) {
	// alias = ((Container) parent).getName() + "." + prs.getName();
	// } else if (parent instanceof ContainerTemplate) {
	// log.error("ProcessingResourceSpecification " + prs.getId()
	// + " is not supported because ContainerTemplates are not yet supported.");
	// } else {
	// log.error("ProcessingResourceSpecification " + prs.getId()
	// + " is not supported because its parent types cannot be resolved.");
	// }
	//
	// if (null != alias) {
	// Entity entity = domainFactory.createResource();
	// entity.setIdentifier(prs.getId());
	// entity.setAlias(alias);
	// resources.add((Resource) entity);
	// }
	// }

	@Override
	public EntityMapping requestDoF(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final String modelLocation = request.getModelLocation();
		final ArrayList<EObject> queryResults = new ArrayList<EObject>();

		return response;
	}

	@Override
	public EntityMapping requestEntities(final EntityMapping request) {

		final EntityMapping response = EcoreUtil.copy(request);
		
		KiekerManager manager = new KiekerManager(request);
		manager.run();
		final Trace queryResults = manager.getAnalysisResult();
		processKiekerResults(response,queryResults);

		return response;
	}

	@Override
	public EntityMapping requestProbes(final EntityMapping request) {
		// Load and validate Model

		final EntityMapping response = EcoreUtil.copy(request);
		// processProbesForServices(response.getServices());
		// processProbesForResources(response.getResources());
		return response;
	}

	@Override
	public boolean supportsDoFRequests() {
		return true;
	}

	@Override
	public boolean supportsEntityRequests() {
		return true;
	}

	@Override
	public boolean supportsProbeRequests() {
		return true;
	}

	@Override
	public void addQueryProgressListener(QueryProgressListener arg0) {
		// TODO Auto-generated method stub

	}
	
	private void processKiekerResults(EntityMapping response, Trace kiekerResults) {
		List<ResourceRecord> resources = kiekerResults.getResources();
		List<EventRecord> events = kiekerResults.getEvents();
		
		final Map<String, String> resourceIdentifiers = new HashMap<>();
		final Map<String, String> serviceIdentifiers = new HashMap<>();
		
		 final EList<Entity> entities = response.getEntities();
		 for (ResourceRecord record: resources ) {
			 resourceIdentifiers.put(record.getIdentifier().getRessourceName(), "");
	 	 }
		 for (EventRecord record: events ) {
			 if (record.getIdentifier().getInterfaceID() != null && !record.getIdentifier().getInterfaceID().equals("")){
				 serviceIdentifiers.put(record.getIdentifier().getComponentID()+"."+record.getIdentifier().getInterfaceID(), "");
			 } else {
				 serviceIdentifiers.put(record.getIdentifier().getComponentID(), "");
			 }
	 	 }
		 
		 for (String identifier: resourceIdentifiers.keySet() ) {
			 Entity entity = domainFactory.createEntity();
			 entity.setEntityType("RESOURCE");
			 entity.setIdentifier(identifier);
			 entities.add(entity);
	 	 }
		 for (String identifier: serviceIdentifiers.keySet() ) {
			 Entity entity = domainFactory.createEntity();
			 entity.setEntityType("SERVICE");
			 entity.setIdentifier(identifier);
			 entities.add(entity);
	 	 }
	}

}
