package tools.descartes.dql.connector.kieker.structures.interfaces;


import tools.descartes.dql.connector.kieker.structures.AbstractCollectorMap;
import tools.descartes.dql.connector.kieker.structures.AbstractValue;
import tools.descartes.dql.connector.kieker.structures.classes.ClassMap;
import tools.descartes.dql.connector.kieker.structures.classes.ClassMapKey;
import tools.descartes.dql.connector.kieker.structures.Trace;
import tools.descartes.dql.connector.kieker.structures.identifier.ComponentIdentifier;
import tools.descartes.dql.connector.kieker.structures.records.EventRecord;



/**
 * 
 * @author Jack
 *
 */
public class InterfaceMap extends AbstractCollectorMap<InterfaceMapKey,InterfaceMapValue>{

	private static final long serialVersionUID = -6854774563826952290L;
	private static InterfaceMap instance;

	private InterfaceMap(){};


	/**
	 * 
	 * @return Singleton instance
	 */
	public static synchronized InterfaceMap getInstance () {
		if (InterfaceMap.instance == null) {
			InterfaceMap.instance = new InterfaceMap();
		}
		return InterfaceMap.instance;
	}


	@Override
	protected Object isUpdateValue(Object key, Object newValue) {
		InterfaceMapValue newInterfaceValue = null;
		if (newValue instanceof InterfaceMapValue) {
			newInterfaceValue = (InterfaceMapValue) newValue;
			InterfaceMapValue oldValue = (InterfaceMapValue) get(key);
			long timestamp = newInterfaceValue.getCurrentTimestamp();
			long exeTime = newInterfaceValue.getCurrentData();
			InterfaceMapValue enhancedValue = oldValue.enhance(timestamp, exeTime);
			return enhancedValue;
		} 
		return null;
	}



	@Override
	public void pushToTrace(Trace trace) {
		for (Entry<InterfaceMapKey, InterfaceMapValue> entry : entrySet()) {

			ClassMapKey classKey = ClassMap.getInstance().getKeyViaID(entry.getKey().getClassID());
			String classID= classKey.getClassID();
			String hostID= classKey.getHostID();
			String interfaceID =entry.getKey().getInterfaceID();
			String requestID = entry.getKey().getRequestID();
			
			for (AbstractValue<Long>.TimeDataPair<Long, Long> item : entry.getValue().getArrayList()) {			
				long timestamp = item.getTime();
				long exTime = item.getData();
				trace.addEvent(new EventRecord(ComponentIdentifier.createInterfaceComponentIdentifier(hostID, classID,interfaceID), timestamp, exTime,requestID));	
			}

		}
		
	}


}
